"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var globby_1 = __importDefault(require("globby"));
var jest_snapshot_1 = require("jest-snapshot");
var path_1 = __importDefault(require("path"));
var SnapshotFetcher = /** @class */ (function () {
    function SnapshotFetcher() {
    }
    SnapshotFetcher.getSnapshots = function () {
        var snapFiles = SnapshotFetcher.getFilePaths();
        return SnapshotFetcher.getSnapsFromFiles(snapFiles);
    };
    SnapshotFetcher.getFilePaths = function () {
        var fullPatterns = SnapshotFetcher.patterns;
        var filePaths = globby_1.default
            .sync(fullPatterns, { dot: true, nodir: true })
            .map(function (filePath) { return path_1.default.relative(process.cwd(), filePath); });
        if (filePaths.length === 0) {
            console.error("No matching files. Patterns tried: " + fullPatterns.join(" "));
            process.exitCode = 2;
        }
        return filePaths;
    };
    SnapshotFetcher.getSnapsFromFiles = function (filePaths) {
        return filePaths
            .map(SnapshotFetcher.getSnapsFromFile)
            .reduce(function (acc, cur) { return acc.concat(cur); }, []);
    };
    SnapshotFetcher.getSnapsFromFile = function (filePath) {
        var absolutePath = path_1.default.resolve(filePath);
        var packageName = SnapshotFetcher.getPackageName(filePath);
        var state = new jest_snapshot_1.SnapshotState(absolutePath, {
            snapshotPath: absolutePath
        });
        var data = state._snapshotData;
        var keys = Object.keys(data);
        var snaps = keys.filter(SnapshotFetcher.isMockshotSnap).map(function (key) { return ({
            key: key,
            packageName: packageName,
            filePath: filePath,
            data: JSON.parse(state._snapshotData[key])
        }); });
        return snaps;
    };
    SnapshotFetcher.getPackageName = function (snapFilePath) {
        var packageJsonPath = SnapshotFetcher.findPackageJson(snapFilePath);
        return packageJsonPath
            ? SnapshotFetcher.getPackageNameFromFile(packageJsonPath)
            : undefined;
    };
    SnapshotFetcher.findPackageJson = function (snapFilePath) {
        var dirName = snapFilePath;
        while (dirName !== ".") {
            dirName = path_1.default.dirname(dirName);
            var maybePackageJson = path_1.default.join(dirName, "package.json");
            if (fs_1.default.existsSync(maybePackageJson)) {
                return maybePackageJson;
            }
        }
        return null;
    };
    SnapshotFetcher.getPackageNameFromFile = function (packageJsonPath) {
        var packageJsonContent = fs_1.default.readFileSync(packageJsonPath).toString();
        return JSON.parse(packageJsonContent).name;
    };
    SnapshotFetcher.isMockshotSnap = function (key) {
        return key.indexOf("[mockshot]") !== -1;
    };
    SnapshotFetcher.patterns = [
        "**/__snapshots__/*.snap",
        "!**/node_modules/**",
        "!./.{git,svn,hg}/**"
    ];
    return SnapshotFetcher;
}());
exports.SnapshotFetcher = SnapshotFetcher;
//# sourceMappingURL=SnapshotFetcher.js.map