"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var util_1 = __importDefault(require("util"));
var ApiMockMatcher_1 = require("../matchers/ApiMockMatcher");
var base_1 = require("./base");
var methodParameter = "url";
var ApiGenerator = /** @class */ (function (_super) {
    __extends(ApiGenerator, _super);
    function ApiGenerator() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.parsed = {};
        return _this;
    }
    ApiGenerator.prototype.generate = function (getFile, allSnapshots) {
        var _this = this;
        var snapshots = allSnapshots.filter(function (snap) {
            return snap.key.includes(ApiMockMatcher_1.ApiSnapshotTag);
        });
        var parsed = this.parse(snapshots);
        var fileDeclaration = getFile("API.ts");
        fileDeclaration
            .addClass({ name: "API" })
            .setIsExported(true)
            .addMethods(this.getMethodsFrom(parsed))
            .map(function (methodDeclaration) {
            methodDeclaration.setBodyText(_this.getSwitchStatement(methodParameter, parsed[methodDeclaration.getName()]));
        });
        fileDeclaration.addInterfaces(this.getInterfacesFrom(parsed));
        return fileDeclaration;
    };
    ApiGenerator.prototype.getMethodsFrom = function (parsed) {
        return Object.keys(parsed).map(function (method) { return ({
            isStatic: true,
            parameters: [{ name: methodParameter, type: "T" }],
            name: method + "<T extends keyof " + method + "Responses>",
            returnType: method + "Responses[T]"
        }); });
    };
    ApiGenerator.prototype.getInterfacesFrom = function (parsed) {
        return Object.keys(parsed).map(function (method) { return ({
            name: method + "Responses",
            isExported: true,
            properties: Object.keys(parsed[method]).map(function (url) {
                var types = Object.keys(parsed[method][url])
                    .map(function (type) { return "\"" + type + "\": any"; })
                    .join();
                return { name: "\"" + url + "\"", type: "{" + types + "}" };
            })
        }); });
    };
    ApiGenerator.prototype.validate = function (snapshot) {
        var key = snapshot.key, snap = snapshot.data;
        if (!this.isHttpMethodValid(snap.httpMethod)) {
            throw Error("Invalid http method '" + snap.httpMethod + "' in snapshot '" + key + "'");
        }
        if (!snap.url) {
            throw Error("Missing property url in snapshot '" + key + "'");
        }
        if (!snap.mockName) {
            throw Error("Missing property mockName in snapshot '" + key + "'");
        }
        if (!snap.mock) {
            throw Error("Missing property mock in snapshot '" + key + "'");
        }
        if (!snap.mock.statusCode) {
            throw Error("Missing property statusCode in snapshot '" + key + "'");
        }
    };
    ApiGenerator.prototype.fillMissingPath = function (snap) {
        if (!lodash_1.has(this.parsed, snap.httpMethod)) {
            this.parsed[snap.httpMethod] = {};
        }
        if (!lodash_1.has(this.parsed[snap.httpMethod], snap.url)) {
            this.parsed[snap.httpMethod][snap.url] = {};
        }
    };
    ApiGenerator.prototype.parse = function (snapshots) {
        var _this = this;
        snapshots.forEach(function (snapshot) {
            var snap = snapshot.data;
            _this.validate(snapshot);
            _this.fillMissingPath(snap);
            if (lodash_1.has(_this.parsed[snap.httpMethod][snap.url], snap.mockName)) {
                throw Error("Snapshot duplication: snapshot with the same httpMethod, URL and mockName already exists '" + snapshot.key + "'");
            }
            _this.parsed[snap.httpMethod][snap.url][snap.mockName] = snap.mock;
        });
        return this.parsed;
    };
    ApiGenerator.prototype.isHttpMethodValid = function (method) {
        return lodash_1.includes(["post", "get", "put", "delete", "patch"], method);
    };
    ApiGenerator.prototype.getSwitchStatement = function (methodParameter, options) {
        return function (writer) {
            return writer
                .write("switch (" + methodParameter + ")")
                .block(function () {
                return Object.keys(options).map(function (key) {
                    return writer
                        .write("case \"" + key + "\":")
                        .indentBlock(function () {
                        return writer.write("return " + util_1.default.inspect(options[key], { depth: null }));
                    });
                });
            });
        };
    };
    return ApiGenerator;
}(base_1.MockGenerator));
exports.ApiGenerator = ApiGenerator;
//# sourceMappingURL=ApiGenerator.js.map