"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var json_stable_stringify_1 = __importDefault(require("json-stable-stringify"));
var path_1 = __importDefault(require("path"));
var ClassMockMatcher_1 = require("../matchers/ClassMockMatcher");
var base_1 = require("./base");
var ClassGenerator = /** @class */ (function (_super) {
    __extends(ClassGenerator, _super);
    function ClassGenerator() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.mockDef = {};
        return _this;
    }
    ClassGenerator.prototype.generate = function (getFile, allSnapshots) {
        var _this = this;
        var snapshots = allSnapshots.filter(function (snap) {
            return snap.key.includes(ClassMockMatcher_1.ClassSnapshotTag);
        });
        snapshots.forEach(function (snap) {
            return _this.parseSingleMock(snap.data, snap.packageName);
        });
        var classNames = Object.keys(this.mockDef);
        classNames.forEach(function (className) {
            var mockClassName = _this.getMockClassName(className);
            var myClassFile = getFile(className + ".ts");
            var classDeclaration = myClassFile.addClass({
                name: mockClassName
            });
            classDeclaration.setIsExported(true);
            var methods = _this.mockDef[className];
            var methodNames = Object.keys(methods);
            methodNames.forEach(function (methodName) {
                var mocks = methods[methodName];
                var mockNames = Object.keys(mocks);
                var mockTypes = mockNames.map(function (value) {
                    return "\"" + value + "\"";
                });
                var types = mockTypes.join(" | ");
                var method = classDeclaration.addMethod({
                    isStatic: true,
                    parameters: [{ name: "mock", type: types }],
                    name: methodName,
                    returnType: "any"
                });
                method.setBodyText(function (writer) {
                    return writer.write("switch (mock)").block(function () {
                        mockNames.forEach(function (mockName) {
                            writer.write("case \"" + mockName + "\":").indentBlock(function () {
                                writer.write("return " + json_stable_stringify_1.default(mocks[mockName], { space: "  " }));
                            });
                        });
                        writer.write("default:").indentBlock(function () {
                            writer.write("throw Error(\"Unknown mock: \"+mock);");
                        });
                    });
                });
            });
        });
    };
    ClassGenerator.prototype.parseSingleMock = function (snapshot, packageName) {
        var fullClassName = this.getFullClassName(snapshot.className, packageName);
        var classDef = this.mockDef[fullClassName];
        if (!classDef) {
            classDef = {};
            this.mockDef[fullClassName] = classDef;
        }
        var methodDef = classDef[snapshot.methodName];
        if (!methodDef) {
            methodDef = {};
            classDef[snapshot.methodName] = methodDef;
        }
        var mockName = methodDef[snapshot.mockName];
        if (!mockName) {
            methodDef[snapshot.mockName] = snapshot.mock;
        }
        else {
            throw Error("Duplicate mock name on class: " + fullClassName + " for method " +
                snapshot.methodName +
                ": " +
                snapshot.mockName);
        }
    };
    ClassGenerator.prototype.getFullClassName = function (className, packageName) {
        return packageName ? path_1.default.join(packageName, className) : className;
    };
    ClassGenerator.prototype.getMockClassName = function (fullClassName) {
        return path_1.default.basename(fullClassName) + "Mocks";
    };
    return ClassGenerator;
}(base_1.MockGenerator));
exports.ClassGenerator = ClassGenerator;
//# sourceMappingURL=ClassGenerator.js.map