"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var json_stable_stringify_1 = __importDefault(require("json-stable-stringify"));
var path_1 = __importDefault(require("path"));
var ts_simple_ast_1 = require("ts-simple-ast");
var ClassMockMatcher_1 = require("../matchers/ClassMockMatcher");
var base_1 = require("./base");
var parsers_1 = require("./parsers");
var ClassSpyGenerator = /** @class */ (function (_super) {
    __extends(ClassSpyGenerator, _super);
    function ClassSpyGenerator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ClassSpyGenerator.prototype.generate = function (getFile, allSnapshots) {
        var _this = this;
        var classSnapshots = this.filterSnapshots(allSnapshots);
        var tree = parsers_1.parseClassSnapshots(classSnapshots);
        Object.keys(tree).forEach(function (filePath) {
            var file = getFile(_this.getFilePath(filePath));
            var _a = tree[filePath], className = _a.className, classContent = _a.classContent;
            _this.writeSpy(file, className, classContent);
        });
    };
    ClassSpyGenerator.prototype.filterSnapshots = function (allSnapshots) {
        return allSnapshots.filter(function (snap) { return snap.key.includes(ClassMockMatcher_1.ClassSnapshotTag); });
    };
    ClassSpyGenerator.prototype.writeSpy = function (file, className, classTree) {
        this.writeMockshotMockInterface(file);
        this.writeSpyInterface(file, className, classTree);
        this.writeClassTree(file, classTree);
        this.writeGetSpyFunction(file);
        this.writeClassSpy(file, className, classTree);
    };
    ClassSpyGenerator.prototype.writeMockshotMockInterface = function (file) {
        file.addInterface({
            name: "MockshotMock<P, T = {}>",
            extends: ["jest.Mock"],
            properties: [
                { name: "mockImplementation(mockName: P)", type: "jest.Mock<T>" },
                {
                    name: "mockImplementation(fn: (...args: any[]) => any)",
                    type: "jest.Mock<T>"
                },
                { name: "mockImplementationOnce(mockName: P)", type: "jest.Mock<T>" },
                {
                    name: "mockImplementationOnce(fn: (...args: any[]) => any)",
                    type: "jest.Mock<T>"
                }
            ]
        });
    };
    ClassSpyGenerator.prototype.writeSpyInterface = function (file, className, classTree) {
        file.addInterface({
            name: className + "Spy",
            properties: this.getMethodsTypes(classTree)
        });
    };
    ClassSpyGenerator.prototype.writeClassTree = function (file, classTree) {
        file.addVariableStatement({
            declarationKind: ts_simple_ast_1.VariableDeclarationKind.Const,
            declarations: [{ name: "classTree", initializer: json_stable_stringify_1.default(classTree) }]
        });
    };
    ClassSpyGenerator.prototype.writeGetSpyFunction = function (file) {
        file.addFunction({
            name: "getSpy<P extends string>",
            parameters: [{ name: "methodName", type: "string" }],
            returnType: "MockshotMock<P>",
            bodyText: function (writer) {
                writer.writeLine("const newSpy = jest.fn() as MockshotMock<P>;");
                writer.writeLine("const getMock = mockName => classTree[methodName][mockName].mock;");
                writer.writeLine("const { mockImplementation, mockImplementationOnce } = newSpy;");
                writer.writeLine("newSpy.mockImplementation = mockName =>");
                writer.writeLine('  typeof mockName === "string"');
                writer.writeLine("    ? mockImplementation(() => getMock(mockName))");
                writer.writeLine("    : mockImplementation(mockName);");
                writer.writeLine("newSpy.mockImplementationOnce = mockName =>");
                writer.writeLine('  typeof mockName === "string"');
                writer.writeLine("    ? mockImplementationOnce(() => getMock(mockName))");
                writer.writeLine("    : mockImplementationOnce(mockName);");
                writer.writeLine("return newSpy;");
            }
        });
    };
    ClassSpyGenerator.prototype.writeClassSpy = function (file, className, classTree) {
        file.addVariableStatement({
            declarationKind: ts_simple_ast_1.VariableDeclarationKind.Const,
            isExported: true,
            declarations: [
                {
                    name: this.getSpyClassName(className),
                    type: this.getSpyInterfaceName(className),
                    initializer: function (writer) {
                        writer.writeLine("{");
                        Object.keys(classTree).map(function (methodName) {
                            return writer.writeLine("    " + methodName + ": getSpy(\"" + methodName + "\"),");
                        });
                        writer.writeLine("}");
                    }
                }
            ]
        });
    };
    ClassSpyGenerator.prototype.getMethodsTypes = function (classTree) {
        var _this = this;
        return Object.keys(classTree).map(function (methodName) {
            var methodTree = classTree[methodName];
            var methodArgUnionType = _this.getMethodArgUnionType(methodTree);
            return { name: methodName, type: "MockshotMock<" + methodArgUnionType + ">" };
        });
    };
    ClassSpyGenerator.prototype.getMethodArgUnionType = function (methodTree) {
        return Object.keys(methodTree)
            .map(function (mockName) { return "\"" + mockName + "\""; })
            .join(" | ");
    };
    ClassSpyGenerator.prototype.getSpyInterfaceName = function (className) {
        return className + "Spy";
    };
    ClassSpyGenerator.prototype.getSpyClassName = function (className) {
        return className.charAt(0).toLowerCase() + className.slice(1) + "Spy";
    };
    ClassSpyGenerator.prototype.getFilePath = function (filePath) {
        return path_1.default.join("spies", filePath);
    };
    return ClassSpyGenerator;
}(base_1.MockGenerator));
exports.ClassSpyGenerator = ClassSpyGenerator;
//# sourceMappingURL=SpyGenerator.js.map