"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var path_1 = __importDefault(require("path"));
function parseClassSnapshots(snapshots) {
    var classTree = {};
    snapshots.forEach(function (snap) { return addSingleSnapshot(snap, classTree); });
    return classTree;
}
exports.parseClassSnapshots = parseClassSnapshots;
function addSingleSnapshot(snap, classTree) {
    var _a = snap.data, className = _a.className, methodName = _a.methodName, mockName = _a.mockName, mock = _a.mock;
    var mockFilePath = getFilePath(snap);
    var mockPath = [mockFilePath, "classContent", methodName, mockName];
    var meta = { key: snap.key, originFile: snap.filePath };
    if (lodash_1.has(classTree, mockPath)) {
        throw Error("Duplicate mock name on class: " + snap.data.className + " for method " + snap.data.methodName + ": " + snap.data.mockName);
    }
    lodash_1.set(classTree, mockPath, { mock: mock, meta: meta });
    classTree[mockFilePath].className = className;
}
function getFilePath(snap) {
    var packageName = snap.packageName || "";
    return path_1.default.join(packageName, snap.data.className) + ".ts";
}
//# sourceMappingURL=classParser.js.map