"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var url_1 = __importDefault(require("url"));
var constants_1 = require("../constants");
exports.ApiSnapshotTag = "[APISnap]";
function toMatchApiMock(response, mockName) {
    if (mockName === void 0) { mockName = constants_1.Success; }
    var parsedResponse;
    try {
        parsedResponse = parse(response);
    }
    catch (err) {
        return { pass: false, message: function () { return err.message; } };
    }
    var snapshot = __assign({ mockName: mockName }, parsedResponse);
    var snapshotTag = constants_1.MockshotTag + " " + exports.ApiSnapshotTag + " [" + parsedResponse.httpMethod + " " + parsedResponse.url + " " + mockName + "]";
    return {
        pass: undefined === expect(snapshot).toMatchSnapshot(snapshotTag),
        message: function () { return "expected " + snapshot + " to match tag " + snapshotTag; }
    };
}
exports.toMatchApiMock = toMatchApiMock;
function parse(response) {
    if (response.config &&
        response.config.method &&
        response.config.url &&
        response.status &&
        response.data) {
        // response is done with axios library (https://www.npmjs.com/package/axios)
        return {
            httpMethod: response.config.method.toLowerCase(),
            url: getPathname(response.config.url),
            mock: { statusCode: response.status, body: response.data }
        };
    }
    else if (response.status && response.req && response.req.method) {
        // response is done with chai library (https://www.google.it.on.your/own)
        return {
            httpMethod: response.req.method.toLowerCase(),
            url: getPathname(response.request.url),
            mock: { statusCode: response.status, body: response.body }
        };
    }
    else {
        throw Error("The response type is not supported.\n       we're supporting only the usage of: chai & axios.\n       You can submit an issue on https://github.com/iqoqo/mockshot/issues to add support for another library");
    }
}
function getPathname(responseUrl) {
    var pathname = url_1.default.parse(responseUrl).pathname;
    if (pathname === undefined) {
        throw new Error("could not parse path name for url " + responseUrl);
    }
    return pathname;
}
//# sourceMappingURL=ApiMockMatcher.js.map