"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var constants_1 = require("../constants");
exports.ClassSnapshotTag = "[ClassSnap]";
function toMatchClassMock(mock, mockedClassOrClassName, methodName, mockNameOrIgnoreFields, maybeIgnoreFields) {
    var className = typeof mockedClassOrClassName === "string"
        ? mockedClassOrClassName
        : mockedClassOrClassName.constructor.name;
    var mockName = constants_1.Success;
    var ignoreFields = [];
    if (typeof mockNameOrIgnoreFields === "string") {
        mockName = mockNameOrIgnoreFields;
    }
    else if (Array.isArray(mockNameOrIgnoreFields)) {
        ignoreFields = mockNameOrIgnoreFields;
    }
    if (Array.isArray(maybeIgnoreFields)) {
        ignoreFields = maybeIgnoreFields;
    }
    return toMatchClassMockImplementation(this, mock, className, methodName, mockName, ignoreFields);
}
exports.toMatchClassMock = toMatchClassMock;
function toMatchClassMockImplementation(self, mock, className, methodName, mockName, ignoredKeyPaths) {
    var snapshotTag = constants_1.MockshotTag + " " + exports.ClassSnapshotTag + " [[" + className + " " + methodName + " " + mockName + "]]";
    var snapshotName = self.currentTestName + ": " + snapshotTag + " 1";
    var currentSnapshot = self.snapshotState._snapshotData[snapshotName];
    var mockClone = lodash_1.cloneDeep(mock);
    if (ignoredKeyPaths && currentSnapshot) {
        var parsedSnapshot_1 = JSON.parse(currentSnapshot);
        ignoredKeyPaths.forEach(function (keyPath) {
            var val = lodash_1.get(parsedSnapshot_1.mock, keyPath);
            var target = lodash_1.get(mockClone, keyPath);
            if (val &&
                target &&
                (typeof val === typeof target || isTimestamp(target))) {
                lodash_1.set(mockClone, keyPath, val);
            }
        });
    }
    var snapshot = {
        className: className,
        methodName: methodName,
        mockName: mockName,
        mock: mockClone
    };
    return {
        pass: undefined === expect(snapshot).toMatchSnapshot(snapshotTag),
        message: function () { return "expected " + snapshot + " to match tag " + snapshotTag; }
    };
}
var isTimestamp = function (target) { return "object" === typeof target && Date.parse(target); };
//# sourceMappingURL=ClassMockMatcher.js.map